'==========================================================================================================================
' Title: Yahoo Weather Device
' Author: Osler
' Language: VBScript
' Description: A script to gather weather data from the Yahoo RSS feed. Provides current weather conditions and a 1 day
' forecast. WeatherNumber and ForecastWeatherNumber correspond to one of the included .gif files to provide an image
' representation of the current or forecasted weather. All images are from Yahoo's website. Please see the terms and
' conditions of using the YahooWeather RSS feed at http://developer.yahoo.com/weather/
' Device Name: CurrentWeather
' Properties:			CWZipCode (should be populated with your 5-digit zipcode prior to running script)
'				CWUnits	(should be populated with either f for Farenheight or c for Celsius prior to running script)
'				CWWindChill
'				CWWindSpeed
'				CWWindDirection
'				CWHumidity
'				CWBarometer
'				CWRising
'				CWWeather
'				CWTemperature
'				CWLocation
'				CWTime
'				CWWeatherNumber
'				CWDay
'				CWDate
'				CWHigh
'				CWLow
'				CWForecastDay
'				CWForecastDate
'				CWForecastLow
'				CWForecastHigh
'				CWForecastWeather
'				CWForecastWeatherNumber
'==========================================================================================================================
On Error Resume Next

Dim XMLHTTP
Dim URL
Dim ZipCode
Dim Units
Dim I
Dim WeatherXML
Dim xmlData
Dim City, State, Country
Dim Units_Temp, Units_Dist, Units_Pres, Units_Spd
Dim WindChill, WindDirection, WindSpeed
Dim Humidity, Visibility, Pressure, Rising
Dim Sunrise, Sunset
Dim Latitude, Longitude
Dim Condition, Code, Temperature, LastUpdate
Dim Forecast(1,5)

URL = "http://xml.weather.yahoo.com/forecastrss"
ZipCode = GetPropertyValue("CurrentWeather.CWZipCode")
Units = GetPropertyValue("CurrentWeather.CWUnits")
URL = URL & "?p=" & ZipCode & "&u=" & Units

Main

Private Sub Main()
	GetXMLData
	If WeatherXML <> "" Then
		ParseXMLData
		SetHouseBotValues
	End If
End Sub

Private Sub GetXMLData()
	Set XMLHTTP = CreateObject("microsoft.xmlhttp")
	XMLHTTP.Open "GET", URL, True
	XMLHTTP.send
	I = 0
	Do
		Sleep(500)
		If XMLHTTP.readyState = 4 Then
			Exit Do
		End If
		If I = 8 Then
			Exit Sub
		End If
		I = I + 1
	Loop
	WeatherXML = XMLHTTP.responseText
	Set XMLHTTP = Nothing
End Sub

Private Sub ParseXMLData()
	On Error Resume Next
	Set xmlData = CreateObject("Microsoft.XMLDom")
	xmlData.async = "false"
	xmlData.loadXML(WeatherXML)

	City = xmlData.getElementsByTagName("yweather:location").item(0).getAttribute("city")
	State = xmlData.getElementsByTagName("yweather:location").item(0).getAttribute("region")
	Country = xmlData.getElementsByTagName("yweather:location").item(0).getAttribute("country")
	Units_Temp = xmlData.getElementsByTagName("yweather:units").item(0).getAttribute("temperature")
	Units_Dist = xmlData.getElementsByTagName("yweather:units").item(0).getAttribute("distance")
	Units_Pres = xmlData.getElementsByTagName("yweather:units").item(0).getAttribute("pressure")
	Units_Spd = xmlData.getElementsByTagName("yweather:units").item(0).getAttribute("speed")
	WindChill = xmlData.getElementsByTagName("yweather:wind").item(0).getAttribute("chill") & " " & Units_Temp
	WindSpeed = xmlData.getElementsByTagName("yweather:wind").item(0).getAttribute("speed") & " " & Units_Spd
	WindDirection = xmlData.getElementsByTagName("yweather:wind").item(0).getAttribute("direction")
	Humidity = xmlData.getElementsByTagName("yweather:atmosphere").item(0).getAttribute("humidity") & " %"
	Visibility = xmlData.getElementsByTagName("yweather:atmosphere").item(0).getAttribute("visibility")
	Pressure = xmlData.getElementsByTagName("yweather:atmosphere").item(0).getAttribute("pressure") & " " & Units_Pres
	Rising = xmlData.getElementsByTagName("yweather:atmosphere").item(0).getAttribute("rising")
	If CInt(Rising) = 0 Then
		Rising = "Steady"
	ElseIf CInt(Rising) = 1 Then
		Rising = "Rising"
	ElseIf CInt(Rising) = 2 Then
		Rising = "Falling"
	End If
	Sunrise = xmlData.getElementsByTagName("yweather:astronomy").item(0).getAttribute("sunrise")
	Sunset = xmlData.getElementsByTagName("yweather:astronomy").item(0).getAttribute("sunset")
	Latitude = xmlData.getElementsByTagName("geo:lat").item(0).text
	Longitude = xmlData.getElementsByTagName("geo:long").item(0).text
	Condition = xmlData.getElementsByTagName("yweather:condition").item(0).getAttribute("text")
	Code = xmlData.getElementsByTagName("yweather:condition").item(0).getAttribute("code")
	If Len(Code) <> 2 Then
		Code = "0" & Code
	End If
	Temperature = xmlData.getElementsByTagName("yweather:condition").item(0).getAttribute("temp") & " " & Units_Temp
	LastUpdate = xmlData.getElementsByTagName("yweather:condition").item(0).getAttribute("date")
	For I = 0 To 1
		Forecast(I, 0) = xmlData.getElementsByTagName("yweather:forecast").item(I).getAttribute("day")
		Forecast(I, 1) = xmlData.getElementsByTagName("yweather:forecast").item(I).getAttribute("date")
		Forecast(I, 2) = xmlData.getElementsByTagName("yweather:forecast").item(I).getAttribute("low")
		Forecast(I, 3) = xmlData.getElementsByTagName("yweather:forecast").item(I).getAttribute("high")
		Forecast(I, 4) = xmlData.getElementsByTagName("yweather:forecast").item(I).getAttribute("text")
		Forecast(I, 5) = xmlData.getElementsByTagName("yweather:forecast").item(I).getAttribute("code")
		If Len(Forecast(I,5)) <> 2 Then
			Forecast(I,5) = "0" & Forcast(I,5)
		End If
	Next
	Set xmlData = Nothing
End Sub

Private Sub SetHouseBotValues()
	Call SetPropertyValue("CurrentWeather.CWVisibility", Visibility)
	Call SetPropertyValue("CurrentWeather.CWWindChill", WindChill)
	Call SetPropertyValue("CurrentWeather.CWWindSpeed", WindSpeed)
	Call SetPropertyValue("CurrentWeather.CWWindDirection", WindDirection)
	Call SetPropertyValue("CurrentWeather.CWHumidity", Humidity)
	Call SetPropertyValue("CurrentWeather.CWBarometer", Pressure)
	Call SetPropertyValue("CurrentWeather.CWRising", Rising)
	Call SetPropertyValue("CurrentWeather.CWWeather", Condition)
	Call SetPropertyValue("CurrentWeather.CWTemperature", Temperature)
	Call SetPropertyValue("CurrentWeather.CWLocation", City & ", " & State & " (" & Country & ")")
	Call SetPropertyValue("CurrentWeather.CWTime", LastUpdate)
	Call SetPropertyValue("CurrentWeather.CWWeatherNumber", "C:\Program Files\HouseBot\Config\Themes\Weather Images\" & Code & ".png")
	Call SetPropertyValue("CurrentWeather.CWDay", Forecast(0,0))
	Call SetPropertyValue("CurrentWeather.CWDate", Forecast(0,1))
	Call SetPropertyValue("CurrentWeather.CWLow", Forecast(0,2) & " " & Units_Temp)
	Call SetPropertyValue("CurrentWeather.CWHigh", Forecast(0,3) & " " & Units_Temp)
	Call SetPropertyValue("CurrentWeather.CWForecastDay", Forecast(1,0))
	Call SetPropertyValue("CurrentWeather.CWForecastDate", Forecast(1,1))
	Call SetPropertyValue("CurrentWeather.CWForecastLow", Forecast(1,2) & " " & Units_Temp)
	Call SetPropertyValue("CurrentWeather.CWForecastHigh", Forecast(1,3) & " " & Units_Temp)
	Call SetPropertyValue("CurrentWeather.CWForecastWeather", Forecast(1,4))
	Call SetPropertyValue("CurrentWeather.CWForecastWeatherNumber", "C:\Program Files\HouseBot\Config\Themes\Weather Images\" & Forecast(1,5) & ".png")
End Sub